@@ -1,25 +1,29 @@
 """a JupyterLite addon for creating the env for xeus-python"""
 import os
-from subprocess import (
-    check_call,
-    DEVNULL
-)
+from subprocess import check_call, run, DEVNULL
 from tempfile import TemporaryDirectory
+import json
 import shutil
 from pathlib import Path
 
-from traitlets import List
+from traitlets import List, Unicode
 
 from empack.file_packager import pack_python_core
 
-from jupyterlite.constants import SHARE_LABEXTENSIONS
-from jupyterlite.addons.federated_extensions import FederatedExtensionAddon
+from jupyterlite.constants import SHARE_LABEXTENSIONS, UTF8
+from jupyterlite.addons.federated_extensions import (
+    FederatedExtensionAddon,
+    ENV_EXTENSIONS,
+)
 
+JUPYTERLITE_XEUS_PYTHON = "@jupyterlite/xeus-python-kernel"
+
+# TODO Make this configurable
 PYTHON_VERSION = "3.10"
 
 CHANNELS = [
     "https://repo.mamba.pm/emscripten-forge",
-    "https://repo.mamba.pm/conda-forge"
+    "https://repo.mamba.pm/conda-forge",
 ]
 PLATFORM = "emscripten-32"
 
@@ -27,6 +31,7 @@ SILENT = dict(stdout=DEVNULL, stderr=DEVNULL)
 
 try:
     from mamba.api import create as mamba_create
+
     MAMBA_PYTHON_AVAILABLE = True
 except ImportError:
     MAMBA_PYTHON_AVAILABLE = False
@@ -57,17 +62,27 @@ class PackagesList(List):
 
 class XeusPythonEnv(FederatedExtensionAddon):
 
-    __all__ = ["pre_build", "post_build"]
+    __all__ = ["post_build"]
+
+    xeus_python_version = Unicode().tag(
+        config=True, description="The xeus-python version to use"
+    )
 
     packages = PackagesList([]).tag(
         config=True,
-        description="A comma-separated list of packages to install in the xeus-python env"
+        description="A comma-separated list of packages to install in the xeus-python env",
     )
 
     @property
     def specs(self):
         """The package specs to install in the environment."""
-        return [f"python={PYTHON_VERSION}", "xeus-python", *self.packages]
+        return [
+            f"python={PYTHON_VERSION}",
+            "xeus-python"
+            if not self.xeus_python_version
+            else f"xeus-python={self.xeus_python_version}",
+            *self.packages,
+        ]
 
     @property
     def prefix_path(self):
@@ -87,10 +102,16 @@ class XeusPythonEnv(FederatedExtensionAddon):
 
         self.orig_config = os.environ.get("CONDARC")
 
-    def pre_build(self, manager):
+    def post_build(self, manager):
         """yield a doit task to create the emscripten-32 env and grab anything we need from it"""
-        # Bail early if there is nothing to do
-        if not self.packages:
+        # Install the jupyterlite-xeus-python ourselves
+        for pkg_json in self.env_extensions(ENV_EXTENSIONS):
+            pkg_data = json.loads(pkg_json.read_text(**UTF8))
+            if pkg_data.get("name") == JUPYTERLITE_XEUS_PYTHON:
+                yield from self.safe_copy_extension(pkg_json)
+
+        # Bail early if there is no extra package to install
+        if not self.packages and not self.xeus_python_version:
             return []
 
         # Create emscripten env with the given packages
@@ -107,29 +128,25 @@ class XeusPythonEnv(FederatedExtensionAddon):
         # Find the federated extensions in the emscripten-env and install them
         root = self.prefix_path / SHARE_LABEXTENSIONS
 
-        if not self.is_sys_prefix_ignored():
-            for pkg_json in self.env_extensions(root):
-                yield from self.copy_one_extension(pkg_json)
+        # Copy federated extensions found in the emscripten-env
+        for pkg_json in self.env_extensions(root):
+            yield from self.safe_copy_extension(pkg_json)
 
         # TODO Currently we're shamelessly overwriting the
-        # python_data.{js,data} into the labextension.
+        # python_data.{js,data} into the jupyterlite-xeus-python labextension.
         # We should really find a nicer way.
         # (make jupyterlite-xeus-python extension somewhat configurable?)
         dest = self.output_extensions / "@jupyterlite" / "xeus-python-kernel" / "static"
 
-        task_dep = ["pre_build:federated_extensions:*"]
-
         for file in ["python_data.js", "python_data.data"]:
             yield dict(
-                task_dep=task_dep,
-                name=f"copy:{file}",
+                name=f"xeus:copy:{file}",
                 actions=[(self.copy_one, [Path(self.cwd.name) / file, dest / file])],
             )
 
         for file in ["xpython_wasm.js", "xpython_wasm.wasm"]:
             yield dict(
-                task_dep=task_dep,
-                name=f"copy:{file}",
+                name=f"xeus:copy:{file}",
                 actions=[
                     (
                         self.copy_one,
@@ -149,7 +166,7 @@ class XeusPythonEnv(FederatedExtensionAddon):
                 base_prefix=self.root_prefix,
                 specs=self.specs,
                 channels=CHANNELS,
-                target_platform=PLATFORM
+                target_platform=PLATFORM,
             )
             return
 
@@ -163,7 +180,7 @@ class XeusPythonEnv(FederatedExtensionAddon):
             return self._create_env_with_config("mamba", channels)
 
         if MICROMAMBA_AVAILABLE:
-            check_call(
+            run(
                 [
                     "micromamba",
                     "create",
@@ -177,6 +194,7 @@ class XeusPythonEnv(FederatedExtensionAddon):
                     *self.specs,
                 ],
                 cwd=self.cwd.name,
+                check=True,
             )
             return
 
@@ -190,27 +208,24 @@ class XeusPythonEnv(FederatedExtensionAddon):
         )
 
     def _create_env_with_config(self, conda, channels):
-        check_call(
-            [
-                conda,
-                "create",
-                "--prefix",
-                self.prefix_path,
-                *channels
-            ],
+        run(
+            [conda, "create", "--yes", "--prefix", self.prefix_path, *channels],
             cwd=self.cwd.name,
+            check=True,
         )
         self._create_config()
-        check_call(
+        run(
             [
                 conda,
                 "install",
+                "--yes",
                 "--prefix",
                 self.prefix_path,
                 *channels,
                 *self.specs,
             ],
             cwd=self.cwd.name,
+            check=True,
         )
 
     def _create_config(self):
@@ -218,18 +233,30 @@ class XeusPythonEnv(FederatedExtensionAddon):
             fobj.write(f"subdir: {PLATFORM}")
         os.environ["CONDARC"] = str(self.prefix_path / ".condarc")
 
-    def post_build(self, manager):
-        """Cleanup"""
-        # Bail early if there is nothing to do
-        if not self.packages:
-            return []
+    def safe_copy_extension(self, pkg_json):
+        """Copy a labextension, and overwrite it
+        if it's already in the output
+        """
+        pkg_path = pkg_json.parent
+        stem = json.loads(pkg_json.read_text(**UTF8))["name"]
+        dest = self.output_extensions / stem
+        file_dep = [
+            p
+            for p in pkg_path.rglob("*")
+            if not (p.is_dir() or self.is_ignored_sourcemap(p.name))
+        ]
 
-        shutil.rmtree(self.cwd.name, ignore_errors=True)
+        yield dict(
+            name=f"xeus:copy:ext:{stem}",
+            file_dep=file_dep,
+            actions=[(self.copy_one, [pkg_path, dest])],
+        )
+
+    def __del__(self):
+        # Cleanup
         shutil.rmtree(self.root_prefix, ignore_errors=True)
 
         if self.orig_config is not None:
             os.environ["CONDARC"] = self.orig_config
         elif "CONDARC" in os.environ:
             del os.environ["CONDARC"]
-
-        return []
